@@ -11,6 +11,8 @@ use File::Slurp;
 #require Module::Install::Repository;
 #require Module::Install::Metadata;
 use Text::FindIndent 0.08;
+use Perl::Meta;
+use App::EUMM::Upgrade;
 
 my $content=read_file('Makefile.PL') or die "Cannot find 'Makefile.PL'";
 if ($content =~ /use inc::Module::Install/) {
@@ -28,44 +30,55 @@ sub process_file {
   my $content=shift;
   my $indentation_type = Text::FindIndent->parse($content,first_level_indent_only=>1);
   my $space_to_use;
+  my $indent_str;
   if ($indentation_type =~ /^[sm](\d+)/) {
     print "Indentation with $1 spaces\n";
     $space_to_use=$1;
+    $indent_str=' 'x$space_to_use;
   } elsif ($indentation_type =~ /^t(\d+)/) {
     print "Indentation with tabs, a tab should indent by $1 characters\n";
     $space_to_use=0;
+    $indent_str="\t";
   } else {
     print "Indentation unknown, will use 4 spaces\n";
     $space_to_use=4;
+    $indent_str=' 'x4;
   }
 
-  sub apply_indent {
-    my $content=shift;
-    my $i_from=shift || die;
-    my $i_to=shift;
-    sub _do_replace {
-      my $spaces=shift;
-      my $i_from=shift;
+  {
+    no strict 'refs';
+    *{__PACKAGE__.'::apply_indent'} = sub {
+      my $content=shift;
+      my $i_from=shift || die;
       my $i_to=shift;
-      my $len=length($spaces);
-      my $l1=int($len/$i_from);
-      if ($i_to==0) {
-        return "\t"x$l1;
-      } else {
-        return " " x ($l1*$i_to);
-      }
-    }
-    $content=~s/^((?:[ ]{$i_from})+)/_do_replace($1,$i_from,$i_to)/emg;
+      *{__PACKAGE__.'::_do_replace'} = sub {
+        my $spaces=shift;
+        my $i_from=shift;
+        my $indent_str=shift;
+        my $len=length($spaces);
+        my $l1=int($len/$i_from);
+        return $indent_str x $l1;
+      };
+    $content=~s/^((?:[ ]{$i_from})+)/_do_replace($1,$i_from,$indent_str)/emg;
     return $content;
+  };
   }
 
   my $compat_layer=<<'EOT';
-sub WriteMakefile1 {  #Written by Alexandr Ciornii, version 0.21. Added by eumm-upgrade.
+sub WriteMakefile1 {  #Compatibility code for old versions of EU::MM. Written by Alexandr Ciornii, version 0.23. Added by eumm-upgrade.
     my %params=@_;
     my $eumm_version=$ExtUtils::MakeMaker::VERSION;
     $eumm_version=eval $eumm_version;
     die "EXTRA_META is deprecated" if exists $params{EXTRA_META};
     die "License not specified" if not exists $params{LICENSE};
+    if ($params{AUTHOR} and ref($params{AUTHOR}) eq 'ARRAY' and $eumm_version < 6.5705) {
+        $params{META_ADD}->{author}=$params{AUTHOR};
+        $params{AUTHOR}=join(', ',@{$params{AUTHOR}});
+    }
+    if ($params{TEST_REQUIRES} and $eumm_version < 6.64) {
+        $params{BUILD_REQUIRES}={ %{$params{BUILD_REQUIRES} || {}} , %{$params{TEST_REQUIRES}} };
+        delete $params{TEST_REQUIRES};
+    }
     if ($params{BUILD_REQUIRES} and $eumm_version < 6.5503) {
         #EUMM 6.5502 has problems with BUILD_REQUIRES
         $params{PREREQ_PM}={ %{$params{PREREQ_PM} || {}} , %{$params{BUILD_REQUIRES}} };
@@ -76,36 +89,13 @@ sub WriteMakefile1 {  #Written by Alexandr Ciornii, version 0.21. Added by eumm-
     delete $params{META_MERGE} if $eumm_version < 6.46;
     delete $params{META_ADD} if $eumm_version < 6.46;
     delete $params{LICENSE} if $eumm_version < 6.31;
-    delete $params{AUTHOR} if $] < 5.005;
-    delete $params{ABSTRACT_FROM} if $] < 5.005;
-    delete $params{BINARY_LOCATION} if $] < 5.005;
 
     WriteMakefile(%params);
 }
 EOT
   my $space=' 'x4;
-  $content=~s/
-  \(\s*\$\]\s*>=\s*5\.005\s*\?\s*(?:\#\#\s*\QAdd these new keywords supported since 5.005\E\s*)?
-  \s+\(\s*ABSTRACT(?:_FROM)?\s*=>\s*'([^'\n]+)',\s*(?:\#\s*\Qretrieve abstract from module\E\s*)?
-  \s+AUTHOR\s*=>\s*'([^'\n]+)'
-  \s*\)\s*\Q: ()\E\s*\),\s+
-  /ABSTRACT_FROM => '$1',\n${space}AUTHOR => '$2',\n/sx;
-
-  $content=~s/
-          \(\s*\$ ExtUtils::MakeMaker::VERSION\s+
-          (?:ge\s+' [\d\._]+ ' \s* | >=?\s*[\d\._]+\s+)\?\s+\(\E \s*
-          ( [^()]+? ) \s*
-          \)\s*\:\s*\(\)\s*\),
-  /$space$1/sxg;
-
-  $content=~s/(WriteMakefile\()(\S)/$1\n$space$2/;
-
-  $content=~s/
-          \(\s*\$\]\s* \Q>=\E \s* 5[\d\._]+ \s* \Q? (\E \s+
-          ( [^()]+? ) \s+
-          \)\s*\:\s*\(\)\s*\),
-  /$1/sxg;
-
+  $content=~s/(WriteMakefile\()(\S)/$1\n$indent_str$2/;
+  $content=remove_conditional_code($content,$indent_str);
   my @param;
 
   my @resourses;
@@ -122,13 +112,32 @@ EOT
     push @resourses,"${space}${space}${space}#repository => 'URL to repository here',";
   }
 
-  if ($content=~/\bVERSION_FROM['"]?\s*=>\s*['"]([^'"\n]+)['"]/) {
+  if ($content=~/\bVERSION_FROM['"]?\s*=>\s*['"]([^'"\n]+)['"]/ || $content=~/\bVERSION_FROM\s*=>\s*q\[?([^\]\n]+)\]/) {
     my $main_file=$1;
     my $main_file_content=eval { read_file($1) };
     if (!$main_file_content) {
       print "Cannot open $main_file\n";
     } else {
-      my @links=Module::Install::Metadata::_extract_bugtracker($main_file_content);
+      my @links=Perl::Meta::extract_bugtracker($main_file_content);
+      if (@links==2) {
+        my $remove;
+        my $dist;
+        foreach my $i (0.. $#links) {
+          my $dist1;
+          if ($links[$i] =~ m#^\Qhttp://rt.cpan.org/NoAuth/ReportBug.html?Queue=\E(\w+)#) {
+            $remove = $i;
+            $dist1 = $1;
+          } elsif ($links[$i] =~ m#^\Qhttp://rt.cpan.org/NoAuth/Bugs.html?Dist=\E(\w+)#) {
+            $dist1 = $1;
+          } else {
+            die "unknown bugtraker type $links[$i]";
+          }
+          if ($dist and $dist1 ne $dist) {
+            die;
+          }
+        }
+        splice(@links,$remove,1) if defined $remove;
+      }
       if (@links==1) {
         my $bt=$links[0];
         print "Bugtracker found: $bt\n";
@@ -137,14 +146,16 @@ EOT
         print "Too many links to bugtrackers found in $main_file\n";
       }
       if ($content !~ /\bLICENSE\s*=>\s*['"]/ and $content !~ /'LICENSE'\s*=>\s*['"]/) {
-        my $l=Module::Install::Metadata::_extract_license($main_file_content);
+        my $l=Perl::Meta::extract_license($main_file_content);
         if ($l) {
           push @param,"    LICENSE => '$l',\n";
+        } else {
+          print "license not found\n";
         }
       }
       if ($content !~ /\bMIN_PERL_VERSION['"]?\s*=>\s*['"]?\d/) {
-        my $version=Module::Install::Metadata::_extract_perl_version($main_file_content) ||
-          Module::Install::Metadata::_extract_perl_version($content);
+        my $version=Perl::Meta::extract_perl_version($main_file_content) ||
+          Perl::Meta::extract_perl_version($content);
         if ($version) {
           push @param,"    MIN_PERL_VERSION => '$version',\n";
         }
@@ -153,7 +164,7 @@ EOT
   } else {
     print "VERSION_FROM not found\n";
     if ($content !~ /\bMIN_PERL_VERSION\s*=>\s*['"\d]/) {
-      my $version=Module::Install::Metadata::_extract_perl_version($content);
+      my $version=Perl::Meta::extract_perl_version($content);
       if ($version) {
         push @param,"    MIN_PERL_VERSION => '$version',\n";
       }
@@ -171,7 +182,7 @@ $res
 EOT
   }
 
-  if ($content !~ /\bBUILD_REQUIRES\s*=>\s*\{/) {
+  if ($content !~ /\bBUILD_REQUIRES['"]?\s*=>\s*\{/) {
     push @param,"    #BUILD_REQUIRES => {\n"."    #},\n";
   }
   
@@ -181,11 +192,18 @@ EOT
     $param=apply_indent($param,4,$space_to_use);
     $param=~s/\s+$/\n/s;
   }
+  my $text2replace = 'WriteMakefile\(';
+  #if ($content =~ /WriteMakefile\(\s*(\%\w+)\s*\);/) {
+  #  my $var_params = $1;
+  #  $text2replace = qr/$var_params\s*=\s*\((?:\s|\n)*/;
+  #}
+  
+  #$content=~s/($text2replace)(\S)/$1\n$indent_str$2/;
   $content=~s/WriteMakefile\s*\(/WriteMakefile1($param/s;
 
   #$content=~s/[\r\n]+$//s;
   $compat_layer="\n\n".apply_indent($compat_layer,4,$space_to_use);
-  $content=~s/(__DATA__ | $ )/$compat_layer$1/sx;
+  $content=~s/(package\s+MY; | __DATA__ | $ )/$compat_layer$1/sx;
   # |
   #
   return $content;
@@ -215,83 +233,6 @@ it (set 'c-basic-offset' to your value):
 (c) Alexandr Ciornii
 =cut
 
-package Module::Install::Metadata;
-#by Adam Kennedy and Alexandr Ciornii
-#See Module::Install for copyright
-
-sub _extract_perl_version {
-	if (
-		$_[0] =~ m/
-		^\s*
-		(?:use|require) \s*
-		v?
-		([\d_\.]+)
-		\s* ;
-		/ixms
-	) {
-		my $perl_version = $1;
-		$perl_version =~ s{_}{}g;
-		return $perl_version;
-	} else {
-		return;
-	}
-}
-
-sub _extract_license {
-	if (
-		$_[0] =~ m/
-		(
-			=head \d \s+
-			(?:licen[cs]e|licensing|copyrights?|legal)\b
-			.*?
-		)
-		(=head\\d.*|=cut.*|)
-		\z
-	/ixms ) {
-		my $license_text = $1;
-		my @phrases      = (
-			'under the same (?:terms|license) as (?:perl|the perl programming language)' => 'perl', 1,
-			'under the terms of (?:perl|the perl programming language) itself' => 'perl', 1,
-			'GNU general public license'         => 'gpl',         1,
-			'GNU public license'                 => 'gpl',         1,
-			'GNU lesser general public license'  => 'lgpl',        1,
-			'GNU lesser public license'          => 'lgpl',        1,
-			'GNU library general public license' => 'lgpl',        1,
-			'GNU library public license'         => 'lgpl',        1,
-			'BSD license'                        => 'bsd',         1,
-			'Artistic license'                   => 'artistic',    1,
-			'GPL'                                => 'gpl',         1,
-			'LGPL'                               => 'lgpl',        1,
-			'BSD'                                => 'bsd',         1,
-			'Artistic'                           => 'artistic',    1,
-			'MIT'                                => 'mit',         1,
-			'proprietary'                        => 'proprietary', 0,
-		);
-		while ( my ($pattern, $license, $osi) = splice(@phrases, 0, 3) ) {
-			$pattern =~ s#\s+#\\s+#g;
-			if ( $license_text =~ /\b$pattern\b/i ) {
-			        return $license;
-			}
-		}
-	} else {
-	        return;
-	}
-}
-
-sub _extract_bugtracker {
-	my @links   = $_[0] =~ m#L<(
-	 \Qhttp://rt.cpan.org/\E[^>]+|
-	 \Qhttp://github.com/\E[\w_]+/[\w_]+/issues|
-	 \Qhttp://code.google.com/p/\E[\w_\-]+/issues/list
-	 )>#gx;
-	my %links;
-	@links{@links}=();
-	@links=keys %links;
-	return @links;
-}
-
-1;
-
 package Module::Install::Repository;
 #by Tatsuhiko Miyagawa
 #See Module::Install::Repository for copyright
